﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reactive.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Media.Capture;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

namespace RxSample.Part5
{
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();

            this.NavigationCacheMode = NavigationCacheMode.Required;

            tb.Text = string.Empty;
            var input = (from evtPattern in Observable.FromEventPattern<TextChangedEventArgs>(tb, "TextChanged")
                         select ((TextBox)evtPattern.Sender).Text.Trim())
                        .Throttle(TimeSpan.FromSeconds(0.5))
                        .DistinctUntilChanged();
            //1
            //input.Subscribe(
            //    txt => Debug.WriteLine("Input text is {0}", txt));

            //2
            input.ObserveOn(SynchronizationContext.Current)
                .Subscribe(
                inputText =>
                {
                    Debug.WriteLine("Search text : {0}", inputText);
                    if (inputText.Length > 3)
                    {
                        wv.Navigate(new Uri("http://en.m.wikipedia.org/wiki/" + inputText));
                    }
                    else
                    {
                        wv.Stop();
                    }
                });

            var navigated =
                (from evtPtn in
                    Observable.FromEventPattern<WebViewNavigationCompletedEventArgs>(wv, "NavigationCompleted")
                 select "NavigationCompleted " + evtPtn.EventArgs.Uri);

            //navigated.Subscribe(evtPattern => Debug.WriteLine("navigated {0}", evtPattern.EventArgs.Uri));

            var navigatting =
                (from evtPtn in
                    Observable.FromEventPattern<WebViewNavigationStartingEventArgs>(wv, "NavigationStarting")
                    select "NavigationStarting " + evtPtn.EventArgs.Uri);
            //navigatting.Subscribe(evtPattern => Debug.WriteLine("navigatting {0}", evtPattern.EventArgs.Uri));

            var navigatFail =
                (from evtPtn in Observable.FromEventPattern<WebViewNavigationFailedEventArgs>(wv, "NavigationFailed")
                    select "NavigationFailed " + evtPtn.EventArgs.Uri);

            //navigatFail.Subscribe(evtPattern => Debug.WriteLine("navigate fail {0}", evtPattern.EventArgs.Uri));

            var navi = navigatting.Merge(navigated).Merge(navigatFail);

            navi.Subscribe(naviStatus => Debug.WriteLine(naviStatus));

            var res = (from inp in input
                       from complete in navi
                       select complete);

            //res.Subscribe(sm =>
            //{
            //    Debug.WriteLine("SelectMany navigated result {0}", sm.EventArgs.Uri);
            //});
            res.Subscribe(sm =>
            {
                Debug.WriteLine("SelectMany navigated result {0}", sm);
            });
        }


        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            // TODO: Prepare page for display here.

            // TODO: If your application contains multiple pages, ensure that you are
            // handling the hardware Back button by registering for the
            // Windows.Phone.UI.Input.HardwareButtons.BackPressed event.
            // If you are using the NavigationHelper provided by some templates,
            // this event is handled for you.
        }
    }
}
